/*------------------------------------------------*/
/* 27aDom/xpath.c                                 */

#include <stdio.h>
#include <libxml/parser.h>
#include <libxml/xpath.h>
#include <libxml/xpathInternals.h>

xmlXPathObjectPtr vyhodnotXPath(char *expression, xmlXPathContextPtr context);
void tiskniVysledek(xmlXPathObjectPtr result);
void tiskniNodeSet(xmlXPathObjectPtr result);

int main(void)
{
    xmlDocPtr doc;
    xmlXPathContextPtr context;
    xmlXPathObjectPtr result;
    int i;
    char *filename = "config.xml";
    char *expression[] = {
        "/",
        "/sx:config",
        "/sx:config/sx:databases/sx:database",
        "//sx:database/@type",
        "count(/sx:config/sx:databases/node())",
        "//sx:database[@type='mysql']/sx:name/text()",
        "concat("
            "'mysql -u ',"
            "//sx:database[@type='mysql']/sx:name/text(),"
            "' -p'," "//sx:database[@type='mysql']/sx:password/text())",
        "boolean(//sx:database[@type='mysql']/@enabled)",
        "boolean(//sx:database[@type='postgres']/@enabled)",
        NULL
    };

    doc = xmlParseFile(filename);
    if (doc == NULL) {
        fprintf(stderr, "Document se nepodařilo naparsovat.\n");
        return 1;
    }

    context = xmlXPathNewContext(doc);
    if (context == NULL) {
        fprintf(stderr, "Nepodařilo se vytvořit context\n");
        return 1;
    }

    xmlXPathRegisterNs(context, (xmlChar *) "sx", (xmlChar *)
               "http://www.sallyx.org/sally/c/linux/xml");

    i=0;
    while (expression[i]) {
        printf("Expression:\n\t%s\n", expression[i]);
        result = vyhodnotXPath(expression[i], context);
        i++;
        if (!result)
            continue;
        tiskniVysledek(result);
        xmlXPathFreeObject(result);

    }

    xmlXPathFreeContext(context);
    xmlFreeDoc(doc);

    return 0;
}

xmlXPathObjectPtr vyhodnotXPath(char *expression, xmlXPathContextPtr context)
{
    xmlXPathCompExprPtr comp;
    xmlXPathObjectPtr result;

    /* result = xmlXPathEvalExpression("expression", context); */
    comp = xmlXPathCompile((xmlChar *) expression);
    if (comp == NULL) {
        fprintf(stderr, "Chyba v xmlXPathCompile()\n");
        return NULL;
    }
    result = xmlXPathCompiledEval(comp, context);

    if (result == NULL) {
        fprintf(stderr, "Chyba v xmlXPathEvalExpression()\n");
        return NULL;
    }

    return result;
}

void tiskniVysledek(xmlXPathObjectPtr result)
{
    /* http://xmlsoft.org/html/libxml-xpath.html#xmlXPathObjectType */
    switch (result->type) {
    case XPATH_BOOLEAN:
        printf("\t = %s\n", result->boolval ? "True" : "False");
        break;
    case XPATH_STRING:
        printf("\t = '%s'\n", result->stringval);
        break;
    case XPATH_NUMBER:
        printf("\t = %f\n", result->floatval);
        break;
    case XPATH_NODESET:
        tiskniNodeSet(result);
        break;
    default:
        printf("not implemented\n");
    }
}

void tiskniNodeSet(xmlXPathObjectPtr result)
{
    xmlNodePtr node;
    int i;
    if (xmlXPathNodeSetIsEmpty(result->nodesetval)) {
        printf("\t = NodeSet JE PRÁZDNÁ");
        return;
    }
    for (i = 0; i < result->nodesetval->nodeNr; i++) {
        node = result->nodesetval->nodeTab[i];
        /* http://xmlsoft.org/html/libxml-tree.html#xmlElementType */
        switch (node->type) {
        case XML_DOCUMENT_NODE:
            printf("\t = /\n");
            break;
        case XML_ELEMENT_NODE:
            if ((long int)node->ns > 0)
                printf("\t = <%s:%s/>\n",
                       node->ns->prefix ? node->
                       ns->prefix : node->ns->href,
                       node->
                       name ? (char *)node->name : "(null)");
            else
                printf("\t = <%s/>\n",
                       node->
                       name ? (char *)node->name : "(null)");
            break;
        case XML_ATTRIBUTE_NODE:
            printf("\t = @%s=%s\n", node->name,
                   node->children[0].content);
            break;
        case XML_TEXT_NODE:
            printf("\t = '%s'\n", node->content);
            break;
        case XML_CDATA_SECTION_NODE:
            printf("\t = <![CDATA[%s]]>'\n", node->content);
            break;
        default:
            printf("?????\n");
        }
    }
}

/*------------------------------------------------*/
