/*------------------------------------------------*/
/* 27bSax/sax-parser.c                            */

#include <string.h>
#include <stdio.h>
#include <stdlib.h>
#include "sax-parser.h"
#include "config.h"

void startDocument(void *user_data)
{
    UserData *data = (UserData *) user_data;
    memset(data, 0, sizeof(UserData));
    data->config = createConfigStructure(2);
    if (!data->config) {
        fprintf(stderr, "Nepodařilo se vytvořit Config\n");
    }
}

void endDocument(void *user_data)
{
    printf("Konec parsování.\n");
}

xmlChar *getAttributeValue(char *name, const xmlChar ** attributes,
               int nb_attributes);

void startElementNs(void *ctx,
            const xmlChar * localname,
            const xmlChar * prefix,
            const xmlChar * URI,
            int nb_namespaces,
            const xmlChar ** namespaces,
            int nb_attributes,
            int nb_defaulted, const xmlChar ** attributes)
{
    UserData *data = (UserData *) ctx;

    if (data->text) {
        free(data->text);
        data->text = NULL;
    }
    if (data->unknown) {
        data->unknownDepth++;
        return;
    }

    if (!data->sx_config && !strcmp((char *)localname, "config")
        && !strcmp((char *)prefix, "sx")) {
        data->sx_namespace = URI;
        data->sx_config = true;
        return;
    }

    if (data->sx_config &&
        !strcmp((char *)localname, "version")
        && !strcmp((char *)URI, (char *)data->sx_namespace)) {
        data->version = true;
        data->sx_config = false;
        return;
    }

    if (data->version &&
        !strcmp((char *)localname, "major")
        && !strcmp((char *)URI, (char *)data->sx_namespace)) {
        data->version_major = true;
        data->version = false;
        return;
    }
    if (data->version &&
        !strcmp((char *)localname, "minor")
        && !strcmp((char *)URI, (char *)data->sx_namespace)) {
        data->version_minor = true;
        data->version = false;
        return;
    }

    if (data->sx_config &&
        !strcmp((char *)localname, "databases")
        && !strcmp((char *)URI, (char *)data->sx_namespace)) {
        data->databases = true;
        data->sx_config = false;
        return;
    }

    if (data->databases &&
        !strcmp((char *)localname, "database")
        && !strcmp((char *)URI, (char *)data->sx_namespace)) {
        data->database = true;
        data->databases = false;
        xmlChar *type = getAttributeValue("type", attributes, nb_attributes);
        if (type) {
            data->config->databases[data->databaseNum].type =
                getDatabaseTypeFromText((char *)type);
            free(type);
        }
        xmlChar *enabled =
            getAttributeValue("enabled", attributes, nb_attributes);
        if (enabled) {
            data->config->databases[data->databaseNum].enabled =
                !strcmp("true", (char *)enabled) ? true : false;
            free(enabled);
        }
        return;
    }

    if (data->database
        && !strcmp((char *)localname, "name")
        && !strcmp((char *)URI, (char *)data->sx_namespace)) {
        data->database_name = true;
        data->database = false;
        return;
    }
    if (data->database
        && !strcmp((char *)localname, "password")
        && !strcmp((char *)URI, (char *)data->sx_namespace)) {
        data->database_password = true;
        data->database = false;
        return;
    }
    if (data->database
        && !strcmp((char *)localname, "schema")
        && !strcmp((char *)URI, (char *)data->sx_namespace)) {
        data->database_schema = true;
        data->database = false;
        return;
    }
    if (data->database
        && !strcmp((char *)localname, "host")
        && !strcmp((char *)URI, (char *)data->sx_namespace)) {
        data->database_host = true;
        data->database = false;
        return;
    }
    if (data->database
        && !strcmp((char *)localname, "port")
        && !strcmp((char *)URI, (char *)data->sx_namespace)) {
        data->database_port = true;
        data->database = false;
        return;
    }
}

void endElementNs(void *ctx,
          const xmlChar * localname,
          const xmlChar * prefix, const xmlChar * URI)
{
    UserData *data = (UserData *) ctx;
    if (data->unknown) {
        data->unknownDepth--;
        if (!data->unknownDepth) {
            data->unknown = false;
        }
        return;
    }

    if (data->database_port) {
        data->config->databases[data->databaseNum].port =
            atoi((char *)data->text);
        data->database_port = false;
        data->database = true;
        return;
    }
    if (data->database_host) {
        if (data->text)
            strncpy(data->config->databases[data->databaseNum].host,
                (char *)data->text,
                sizeof(data->config->
                       databases[data->databaseNum].host));
        data->database_host = false;
        data->database = true;
        return;
    }
    if (data->database_schema) {
        if (data->text)
            strncpy(data->config->databases[data->databaseNum].
                schema, (char *)data->text,
                sizeof(data->config->
                       databases[data->databaseNum].schema));
        data->database_schema = false;
        data->database = true;
        return;
    }
    if (data->database_password) {
        if (data->text)
            strncpy(data->config->databases[data->databaseNum].
                password, (char *)data->text,
                sizeof(data->config->
                       databases[data->databaseNum].password));
        data->database_password = false;
        data->database = true;
        return;
    }
    if (data->database_name) {
        if (data->text)
            strncpy(data->config->databases[data->databaseNum].name,
                (char *)data->text,
                sizeof(data->config->
                       databases[data->databaseNum].name));
        data->database_name = false;
        data->database = true;
        return;
    }

    if (data->database) {
        data->database = false;
        data->databases = true;
        data->databaseNum++;
        return;
    }

    if (data->databases) {
        data->databases = false;
        data->sx_config = true;
        return;
    }

    if (data->version_major) {
        if (data->text)
            strncpy(data->config->version.major,
                (char *)data->text,
                sizeof(data->config->version.major));
        data->version_major = false;
        data->version = true;
        return;
    }

    if (data->version_minor) {
        if (data->text)
            strncpy(data->config->version.minor,
                (char *)data->text,
                sizeof(data->config->version.minor));
        data->version_minor = false;
        data->version = true;
        return;
    }

    if (data->version) {
        data->version = false;
        data->sx_config = true;
        return;
    }
}

void characters(void *ctx, const xmlChar * ch, int len)
{
    UserData *data = (UserData *) ctx;
    if (!data->text) {
        data->text = (xmlChar *) malloc(sizeof(xmlChar) * (len + 1));
        memcpy(data->text, ch, len);
        data->text[len] = '\0';
        return;
    }
    size_t size = strlen((char *)data->text);
    data->text = realloc(data->text, size + len + 1);
    memcpy(data->text + size, ch, len);
    data->text[size + len] = '\0';
}

xmlChar *getAttributeValue(char *name, const xmlChar ** attributes,
               int nb_attributes)
{
    int i;
    const int fields = 5;    /* (localname/prefix/URI/value/end) */
    xmlChar *value;
    size_t size;
    for (i = 0; i < nb_attributes; i++) {
        const xmlChar *localname = attributes[i * fields + 0];
        const xmlChar *prefix = attributes[i * fields + 1];
        const xmlChar *URI = attributes[i * fields + 2];
        const xmlChar *value_start = attributes[i * fields + 3];
        const xmlChar *value_end = attributes[i * fields + 4];
        if (strcmp((char *)localname, name))
            continue;
        size = value_end - value_start;
        value = (xmlChar *) malloc(sizeof(xmlChar) * size + 1);
        memcpy(value, value_start, size);
        value[size] = '\0';
        return value;
    }
    return NULL;
}

/*------------------------------------------------*/
